#pragma once
#ifndef SOFTWARE_INFO_H
#define SOFTWARE_INFO_H

#include <string>

#pragma pack(1)

struct StructSoftwareInfo
{
    unsigned char size;             // Strukturgroesse (gleich Version der Info)

    unsigned short device;          // Geraetetyp

    // Ab der Firmwareversion 04.03.03.00 wird dieser Parameter durch deviceExtended ersetzt.
    unsigned short board;           // Hardware

    // Ab der Firmwareversion 04.03.03.00 wird dieser Parameter durch boardExtended ersetzt.
    unsigned char display;          // Anzeige
#define DISPLAY_NONE                             (0x00)
#define DISPLAY_LEHNER_SWAP                      (0x01)
#define DISPLAY_LEHNER                           (0x02)
#define DISPLAY_NEUMUELLER                       (0x04)
#define DISPLAY_TIMEBOY_2ZOLL                    (0x08)

    unsigned char keypad;           // Tastatur
#define KEYPAD_NONE                              (0x00)
#define KEYPAD_PZE                               (0x01)
#define KEYPAD_BDE                               (0x02)
#define KEYPAD_AE                                (0x04)
#define KEYPAD_PZE_BASIC                         (0x08)
#define KEYPAD_TIMEBOY                           (0x10)
#define KEYPAD_FLEX                              (0x20)
#define KEYPAD_TIMEBOY_MOBIL_PZE                 (0x40)

    unsigned char radio;            // Funk
#define RADIO_NONE                               (0x00)
#define RADIO_BLUETOOTH                          (0x01)
#define RADIO_ZIGBEE                             (0x02)

    unsigned char mmc;              // MM Card / USB Stick
#define MMC_NONE                                 (0x00)
#define MMC_MEMORY                               (0x01)
#define MMC_DISK                                 (0x02)
#define MMC_USBHOST                              (0x04)

    unsigned char modem;            // Mobilfunk
#define MODEM_NONE                               (0x00)
#define MODEM_MC39                               (0x01)
#define MODEM_MC55                               (0x02)
#define MODEM_MODEM_RS9110_N_11_24               (0x04)

    unsigned char rs485;            // RS485 Kommunikation
#define RS485_NONE                               (0x00)
#define RS485_TIP                                (0x01)

    unsigned char tcpip;            // TCP/IP | WLAN
#define TCPIP_NONE                               (0x00)
#define TCPIP_XPORT                              (0x01)
#define TCPIP_MATCHPORT                          (0x02)

    unsigned char usb;              // USB Kommunikation
#define USB_NONE                                 (0x00)
#define USB_TIP                                  (0x01)

    unsigned char access;           // Zutrittskontrolle mit externen Lesertyp
#define ACCESS_NONE                              (0x00)
#define ACCESS_GIS                               (0x01)
#define ACCESS_PHG                               (0x02)
#define ACCESS_PROTECT                           (0x04)
#define ACCESS_IDTRONIC                          (0x08)

    unsigned char dcf77;            // Funkuhr oder Barcode
#define DCF77_NONE                               (0x00)
#define DCF77_HKW                                (0x01)
#define DCF77_BARCODE                            (0x02)
#define DCF77_BARCODE_ZEBEX                      (0x04)

    unsigned char docking;          // Anbindung TimeboyIII
#define DOCKING_NONE                             (0x00)
#define DOCKING_ENABLE                           (0x01)

    unsigned char fingerprint;      // Fingerprintmodul 
#define BIOKEY_NONE                              (0x00)
#define BIOKEY_3000                              (0x01)
#define BIOKEY_4000                              (0x02)
#define BIOKEY_4020                              (0x03)

    unsigned char gps;              // GPS Modul
#define GPS_NONE                                 (0x00)
#define GPS_FURUNO                               (0x01)
#define GPS_LEA                                  (0x02)
#define GPS_MOUSE_LEA                            (0x04)

    unsigned char iomodule;         // Anbindung IO-Modul
#define IOMODULE_NONE                            (0x00)
#define IOMODULE_ENABLE                          (0x01)

    unsigned short transponder;     // Transponderleser (siehe auch transpondeExtended fuer weitere Lesertypen)
    // Ab der Firmwareversion 04.02.03.12 wird dieser Parameter durch transponderExtended ersetzt.
    unsigned char analog_in;        // Analoge Eingaenge
#define ANALOG_IN_NONE                           (0x00)
#define ANALOG_IN_2                              (0x03)

    unsigned char digital_in;       // Digitale Eingaenge
#define DIGITAL_IN_NONE                          (0x00)
#define DIGITAL_IN_2                             (0x03)
#define DIGITAL_IN_6                             (0x3F)
#define DIGITAL_IN_8                             (0xFF)

    unsigned char digital_out;      // Digitale Ausgaenge (Relais)
#define DIGITAL_OUT_NONE                         (0x00)
#define DIGITAL_OUT_2                            (0x03)
#define DIGITAL_OUT_4                            (0x0F)

    unsigned char trace;            // 
#define TRACE_NONE                               (0x00)

    //###########################
    // Ab Version 2
    unsigned char comserver;        // Com-Server Modus
#define COMSERVER_NONE                           (0x00)
#define COMSERVER_ENABLE                         (0x01)

    unsigned char transponder_write;// Beschreiben von Transpondern im Transpondermenue
#define TP_WRITE_NONE                            (0x00)
#define TP_WRITE_ENABLE                          (0x01)

    unsigned char http;             // Versenden von Datensaetzen per HTTP
#define HTTP_NONE                                (0x00)
#define HTTP_ENABLE                              (0x01)

    unsigned char language;         // Firmware benoetigt Textdaten zum Ordnungsgemaessen Betrieb
#define LANGUAGE_NONE                            (0x00)
#define LANGUAGE_ENABLE                          (0x01)

    unsigned char biokey_update;    // Update des Integrierten Biokeymoduls ueber die Firmware moeglich
#define BIOKEY_UPDATE_NONE                       (0x00)
#define BIOKEY_UPDATE_ENABLE                     (0x01)
    //###########################
    // Ab Version 3
    unsigned short setup_options;   // Unterstuetzte Bestandteile des Setups
#define SETUP_OPTION_MASK                        (0x0001) // Maske der unterstuetzten Setupoptionen
#define SETUP_OPTION_ALL                         (0xFFFF)
#define SETUP_OPTION_IOCONTROL                   (0x0001) // Signalverarbeitung

    //###########################
    // Ab Version 4
    unsigned char touch;            // Touchpanel
#define TOUCH_NONE                               (0x00)
#define TOUCH_SMALL                              (0x01)
#define TOUCH_BIG                                (0x02)

    unsigned char sensor;           // Sensoren
#define SENSOR_NONE                              (0x00)
#define SENSOR_VIBRATION                         (0x01)
#define SENSOR_SPEEDUP                           (0x02)

    unsigned char battery;          // Lithium Batterie
#define BATTERY_NO                               (0x00)
#define BATTERY_YES                              (0x01)

    unsigned char charge;           // Ladeschaltung
#define CHARGE_NO                                (0x00)
#define CHARGE_YES                               (0x01)

    //###########################
    // Ab Version 5
    unsigned char log;// Unterstuetzung von Systemlog
#define LOG_NONE                                 (0x00)
    unsigned char payable_options;// Bezahlbare Zusatzoptionen
#define PAYABLE_OPTION_MASK                      (0x01) // Maske der unterstuetzten Optionen
#define PAYABLE_OPTION_NONE                      (0x00)
#define PAYABLE_OPTION_DATA_ON_CARD              (0x01)

    unsigned int transponderExtended; // Transponderleser (Die unteren 16bit sind oben bei transponder definiert)
#define TP_NONE                                  (0x00000000)
#define TP_TSR30                                 (0x00000001)
#define TP_TSR32                                 (0x00000002)
#define TP_PRIMO100                              (0x00000004)
#define TP_MIFARE                                (0x00000008)
#define TP_IBUTTON_EXT                           (0x00000010)
#define TP_SMARTRELAIS                           (0x00000020)
#define TP_NEDAP                                 (0x00000040)
#define TP_IBUTTON_INT                           (0x00000080)
#define TP_HID_PROXKEY                           (0x00000100)
#define TP_MULTI_ISO_OEM_READER                  (0x00000200)
#define TP_LEGIC_ADVANT                          (0x00000400)
#define TP_TSR21                                 (0x00000800)
#define TP_HID_ICLASS                            (0x00001000)
#define TP_TB_MIFARE                             (0x00002000)
#define TP_TB_LEGIC                              (0x00004000)
#define TP_SMARTRELAIS_G2                        (0x00008000)
#define TP_MIFARE_NFC_OEM_READER                 (0x00010000)
#define TP_IDTRONIC_UHF                          (0x00020000)

    //###########################
    // Ab Version 6
    unsigned int deviceExtended; // Gerätetypen (Die unteren 16bit sind oben bei device definiert)
#define DEVICE_PZE                               (0x00000001)
#define DEVICE_ZK                                (0x00000002)
#define DEVICE_BDE                               (0x00000004)
#define DEVICE_MOBIL                             (0x00000008)
#define DEVICE_MDEBOX                            (0x00000010)
#define DEVICE_AE                                (0x00000020)
#define DEVICE_TIMEBOY                           (0x00000040)
#define DEVICE_EXKLUSIV                          (0x00000080)
#define DEVICE_FLEX                              (0x00000100)
#define DEVICE_TIMEBOY_MOBIL_PZE                 (0x00000200)
#define DEVICE_TERMINAL_2_8                      (0x00000400)
#define DEVICE_TERMINAL_4_3                      (0x00000800)
#define DEVICE_IPC_EXTENSION_VARIO_5_7           (0x00001000)
#define DEVICE_IPC_EXTENSION_VARIO_10            (0x00002000)
#define DEVICE_IPC_EXTENSION_Q7                  (0x00004000)
#define DEVICE_IO_BOX                            (0x00008000)
#define DEVICE_ZK_BOX                            (0x00010000)
#define DEVICE_MOBIL_BOX                         (0x00020000)
#define DEVICE_TIMEBOY_TBDOCK                    (0x00040000)
#define DEVICE_MOBIL_TBDOCK                      (0x00080000)
#define DEVICE_ZK_KNOTEN                         (0x00100000)
#define DEVICE_EVO_3_5_UNIVERSAL                 (0x00200000)
#define DEVICE_SAMS_OCL_READER                   (0x00400000)
#define DEVICE_TERMINAL_3_5                      (0x00800000)
#define DEVICE_ZK_READER_PLUS                    (0x01000000)
#define DEVICE_ZK_ZENTRALE                       (0x02000000)

    unsigned int boardExtended; // Boardversionen (Die unteren 16bit sind oben bei board definiert)
#define BOARD_10                                 (0x00000001)
#define BOARD_11                                 (0x00000002)
#define BOARD_12                                 (0x00000004)
#define BOARD_13                                 (0x00000008)
#define BOARD_14                                 (0x00000010)
#define BOARD_20                                 (0x00000020)
#define BOARD_21                                 (0x00000040)
#define BOARD_30                                 (0x00000080)
#define BOARD_40                                 (0x00000100)
#define BOARD_41                                 (0x00000200)
#define BOARD_42                                 (0x00000400)
#define BOARD_34                                 (0x00000800)
#define BOARD_43                                 (0x00001000)
#define BOARD_23                                 (0x00002000)
#define BOARD_24                                 (0x00004000)
#define BOARD_31                                 (0x00008000)
#define BOARD_44                                 (0x00010000)
#define BOARD_15                                 (0x00020000)
#define BOARD_45                                 (0x00040000)
#define BOARD_16                                 (0x00080000)
#define BOARD_22                                 (0x00100000)
#define BOARD_25                                 (0x00200000)
#define BOARD_17                                 (0x00400000)
#define BOARD_26                                 (0x00800000) 
#define BOARD_46                                 (0x01000000)
#define BOARD_47								 (0x02000000)
#define BOARD_18								 (0x04000000)
#define BOARD_27								 (0x08000000)
#define BOARD_48								 (0x10000000)
};

#pragma pack()

class SoftwareInfo
{
public:
    SoftwareInfo();
    virtual ~SoftwareInfo();

public:
    void clear();

    void setStructSoftwareInfo( const StructSoftwareInfo &info );
    StructSoftwareInfo structSoftwareInfo() const;

    StructSoftwareInfo check( StructSoftwareInfo swInfo );
    std::string toString();
    static int toString( const StructSoftwareInfo &swInfo, std::string &info );

private:
    StructSoftwareInfo _softwareInfo;
};

#endif
