/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/
#pragma once
#ifndef DATAFOX_COMMUNICATION_PROTOCOL_CRYPT_H
#define DATAFOX_COMMUNICATION_PROTOCOL_CRYPT_H

// Basisklasse einbinden.
#include "datafox/communication/protocol_base.h"
#include <time.h>
#include <string>

#include "common/crc.h"
#include "datafox/communication/aes_crypt_session.h"

#define IV_MODULE_PROTOCOL_CRYPT                 0x0000
#define IV_FUNCTION_PROTOCOL_PING                0x0000
#define IV_FUNCTION_EXCHANGE_PUBLIC_KEY          0x0046

namespace Datafox
{
    namespace Communication
    {
        /// Basisklasse fuer pure protokollbasierte Kommunikationsaufgaben.
        class Protocol_crypt : public Protocol_base
        {
        public:
            Protocol_crypt( int connection_id = Connection_base::dv_connection_id, uint8_t device_address = dv_device_address );
            virtual ~Protocol_crypt();

            // Speicher fuer Pakete/Daten
        public:
            void reset();

        public:
            uint8_t m_packet_number_previous;
            uint8_t m_packet_data_previous[IV_PACKET_LENGTH_DATA];
            uint16_t m_packet_data_size_previous;

        protected:
            Aes_crypt_session m_aes_crypt_session[32];
        public:
            inline uint8_t current_session_id()
            {
                return ((get_device_address_receiver() < 1 || get_device_address_receiver() > 31) ? 0 : get_device_address_receiver());
            }
            inline Aes_crypt_session &aes_crypt_session()
            {
                return m_aes_crypt_session[current_session_id()];
            }

        public:
            // Passwort fuer die verschluesselte Kommunikation setzen
            int set_communication_password( const char* password, uint8_t size, uint32_t session_timeout, bool global );

        public:
            // Ausfuehren eines Protokollbefehls
            virtual int call( uint16_t context, uint16_t module, uint16_t func, uint8_t *buffer, uint16_t &count, int context_value = -1 );

        protected:
            // Packetdaten ggf. verschluesseln.
            virtual int write_protocol_crypt();
        public:
            // Streamdaten ggf. verschluesseln.
            int write_stream_crypt( std::string &buffer );
        protected:
            // Packetdaten ggf. entschluesseln.
            virtual int read_protocol_crypt();
        public:
            // Streamdaten ggf. entschluesseln.
            virtual int read_stream_crypt( std::string &buffer );

            // Pruefen ob aktuell eine Verschluesselte Session aktiv ist.
            bool is_crypt()
            {
                return (aes_crypt_session().m_crypt_mode > 1);
            }

        public:
            // Austausch eines Session-Key fuer die verschluesselte Kommunikation
            int exchange_session_key( uint8_t device_address );

        };

    } // namespace Communication
} // namespace Datafox

#endif
